package jalview.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import MCview.PDBfile;
import jalview.io.EBIFetchClient;
import MCview.PDBChain;
import jalview.datamodel.Alignment;
import jalview.io.FormatAdapter;
import jalview.datamodel.SequenceI;
import jalview.io.IdentifyFile;
import jalview.analysis.AlignSeq;
import jalview.datamodel.*;

public class SequenceFetcher
    extends JPanel implements Runnable
{
  JInternalFrame frame;
  AlignFrame alignFrame;
  public SequenceFetcher(AlignFrame af)
  {
    alignFrame = af;
    database.addItem("-- Select Database --");
    database.addItem("Uniprot");
    database.addItem("EMBL");
    database.addItem("EMBLCDS");
    database.addItem("PDB");

    try
    {
      jbInit();
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }

    frame = new JInternalFrame();
    frame.setContentPane(this);
    Desktop.addInternalFrame(frame, "Sequence Fetcher", 390, 120);
  }

  private void jbInit()
      throws Exception
  {
    this.setLayout(gridBagLayout1);
    database.setFont(new java.awt.Font("Verdana", Font.PLAIN, 11));
    jLabel1.setFont(new java.awt.Font("Verdana", Font.ITALIC, 11));
    jLabel1.setText(
        "Separate multiple accession ids with semi colon \";\"");
    ok.setText("OK");
    ok.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ok_actionPerformed(e);
      }
    });
    cancel.setText("Cancel");
    cancel.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        cancel_actionPerformed(e);
      }
    });
    textfield.setFont(new java.awt.Font("Verdana", Font.PLAIN, 11));
    textfield.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ok_actionPerformed(e);
      }
    });
    jPanel1.add(ok);
    jPanel1.add(cancel);
    this.add(jLabel1, new GridBagConstraints(0, 0, 2, 1, 0.0, 0.0
                                             , GridBagConstraints.WEST,
                                             GridBagConstraints.NONE,
                                             new Insets(7, 4, 0, 6), 77, 6));
    this.add(textfield, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
                                               , GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(0, 0, 0, 6), 221, 1));
    this.add(jPanel1, new GridBagConstraints(0, 2, 2, 1, 1.0, 1.0
                                             , GridBagConstraints.WEST,
                                             GridBagConstraints.BOTH,
                                             new Insets(7, -2, 7, 12), 241, -2));
    this.add(database, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
                                              , GridBagConstraints.CENTER,
                                              GridBagConstraints.NONE,
                                              new Insets(0, 4, 0, 0), 1, 0));
  }

  JComboBox database = new JComboBox();
  JLabel jLabel1 = new JLabel();
  JButton ok = new JButton();
  JButton cancel = new JButton();
  JPanel jPanel1 = new JPanel();
  JTextField textfield = new JTextField();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  public void cancel_actionPerformed(ActionEvent e)
  {
    try
    {
      frame.setClosed(true);
    }
    catch (Exception ex)
    {}
  }

  public void ok_actionPerformed(ActionEvent e)
  {
    frame.setTitle("Sequence Fetcher - Fetching Sequence from " +
                   database.getSelectedItem());
    database.setEnabled(false);
    textfield.setEnabled(false);
    ok.setEnabled(false);
    cancel.setEnabled(false);

    Thread worker = new Thread(this);
    worker.start();

  }

  public void run()
  {
    String error = null;
    StringBuffer result = new StringBuffer();
      if (database.getSelectedItem().equals("Uniprot")
          || database.getSelectedItem().equals("EMBL")
          || database.getSelectedItem().equals("EMBLCDS"))
      {
        EBIFetchClient dbFetch = new EBIFetchClient();
        String[] reply = dbFetch.fetchData(
            database.getSelectedItem().toString().toLowerCase(
            ) + ":" + textfield.getText(),
            "fasta", "raw");

        for (int i = 0; i < reply.length; i++)
          result.append(reply[i] + "\n");
      }
      else if (database.getSelectedItem().equals("PDB"))
      {
        EBIFetchClient ebi = new EBIFetchClient();
        String[] reply = ebi.fetchData("pdb:" + textfield.getText(), "pdb",
                                       "raw");
        try{
          PDBfile pdbfile = new PDBfile(reply);
          for (int i = 0; i < pdbfile.chains.size(); i++)
          {
            result.append("\n>" +
                          ( (PDBChain) pdbfile.chains.elementAt(i)).sequence.
                          getName() +
                          "\n"
                          +
                          ( (PDBChain) pdbfile.chains.elementAt(i)).sequence.
                          getSequence());
          }
        }catch(Exception ex)// Problem parsing PDB file
        {
          error = "Error retrieving "+textfield.getText()+" from "+database.getSelectedItem();
        }
      }

    if (result.length() > 0 && error==null)
    {
      String format = IdentifyFile.Identify(result.toString(), "Paste");
      SequenceI[] sequences = null;

      if (FormatAdapter.formats.contains(format))
      {
        sequences = new FormatAdapter().readFile(result.toString(), "Paste", format);
        if (sequences != null && sequences.length>0)
        {
          if(alignFrame==null)
          {
            AlignFrame af = new AlignFrame(new Alignment(sequences));
            af.currentFileFormat = format;
            Desktop.addInternalFrame(af, "Cut & Paste input - " + format,
                                     AlignFrame.NEW_WINDOW_WIDTH,
                                     AlignFrame.NEW_WINDOW_HEIGHT);
            af.statusBar.setText("Successfully pasted alignment file");
            try
            {
              af.setMaximum(jalview.bin.Cache.getDefault("SHOW_FULLSCREEN", false));
            }
            catch (Exception ex)
            {}
          }
          else
          {
            for (int i = 0; i < sequences.length; i++)
            {
              alignFrame.viewport.alignment.addSequence(sequences[i]);

              ////////////////////////////
              //Datset needs extension;
              /////////////////////////////
              Sequence ds = new Sequence(sequences[i].getName(),
                                         AlignSeq.extractGaps("-. ",
                  sequences[i].getSequence()),
                                         sequences[i].getStart(),
                                         sequences[i].getEnd());
              sequences[i].setDatasetSequence(ds);
              alignFrame.viewport.alignment.getDataset().addSequence(ds);
            }
            alignFrame.viewport.setEndSeq(alignFrame.viewport.alignment.
                                          getHeight());
            alignFrame.viewport.alignment.getWidth();
            alignFrame.viewport.firePropertyChange("alignment", null,
                alignFrame.viewport.getAlignment().getSequences());

          }

          if (database.getSelectedItem().equals("PDB"))
          {
            for (int i = 0; i < sequences.length; i++)
            {
              PDBEntry entry = new PDBEntry();
              entry.setId(textfield.getText());
              sequences[i].getDatasetSequence().addPDBId(entry);
            }
          }

        }
        else
          error = "Error retrieving "+textfield.getText()+" from "+database.getSelectedItem();



      }
    }

    if(error!=null)
          showErrorMessage(error);

    database.setEnabled(true);
    textfield.setEnabled(true);
    ok.setEnabled(true);
    cancel.setEnabled(true);
    frame.setTitle("Sequence Fetcher");


  }

  void showErrorMessage(String error)
  {
    JOptionPane.showInternalMessageDialog(Desktop.desktop,
         error, "Error Retrieving Data", JOptionPane.WARNING_MESSAGE);
  }
}




